/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Authors: Christian Hitz, digitalSTROM AG <christian.hitz@digitalstrom.com>
             Michael Tross, digitalSTROM AG <michael.tross@digitalstrom.com>
             Reto Flütsch, digitalSTROM AG <reto.fluetsch@digitalstrom.com>
             Sergey Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.
    

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/

#include "device-api-proto.h"
#include "vdc-api-proto.h"
#include "dsm.h"
#include "datamodel.h"

#include <syslog.h>
#include <stdlib.h>
#include <string.h>
#include "registers.h"

#include "messages.pb-c.h"
#include "proputils.h"
#include "util.h"
#include "channelutil.h"

Q_DEFINE_THIS_MODULE(device-api)



void device_api_send_scene(const char *dsuid, Scene scene, bool force)
{
    vdsm_report(LOG_DEBUG,
           "device api send scene: on device %s call %sscene %d\n", dsuid,
           force ? "force " : "", scene);

    vdc_client_t *vdc_client;

    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationCallScene submsg =
            VDCAPI__VDSM__NOTIFICATION_CALL_SCENE__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        submsg.scene = scene;
        submsg.has_scene = 1;
        submsg.force = force;
        submsg.has_force = 1;

        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_CALL_SCENE;
        msg.vdsm_send_call_scene = &submsg;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg, NULL);
    }
}

void device_api_send_undo_scene_number(const char *dsuid, Scene scene)
{
    vdsm_report(LOG_DEBUG, "device api send undo scene number: on device %s call "
           "undo scene number %d\n", dsuid, scene);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationUndoScene submsg =
            VDCAPI__VDSM__NOTIFICATION_UNDO_SCENE__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        submsg.scene = scene;
        submsg.has_scene = 1;

        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_UNDO_SCENE;
        msg.vdsm_send_undo_scene = &submsg;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg, NULL);
    }
}

void device_api_send_undo_scene(const char *dsuid)
{
    vdsm_report(LOG_DEBUG,
           "device api send undo scene: on device %s call undo scene\n", dsuid);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationUndoScene submsg =
            VDCAPI__VDSM__NOTIFICATION_UNDO_SCENE__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_UNDO_SCENE;
        msg.vdsm_send_undo_scene = &submsg;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg, NULL);
    }
}

void device_api_send_save_scene(const char *dsuid, Scene scene)
{
    vdsm_report(LOG_DEBUG,
          "device api send save scene: on device %s save scene %d\n",
          dsuid, scene);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationSaveScene submsg =
            VDCAPI__VDSM__NOTIFICATION_SAVE_SCENE__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        submsg.scene = scene;
        submsg.has_scene = 1;

        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_SAVE_SCENE;
        msg.vdsm_send_save_scene = &submsg;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg, NULL);
    }
}

void device_api_send_local_prio(const char *dsuid, Scene scene)
{
    vdsm_report(LOG_DEBUG, "device api send local prio: on device %s call local prio %d\n", dsuid, scene);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationSetLocalPrio submsg =
            VDCAPI__VDSM__NOTIFICATION_SET_LOCAL_PRIO__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        submsg.scene = scene;
        submsg.has_scene = 1;

        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_SET_LOCAL_PRIO;
        msg.vdsm_send_set_local_prio = &submsg;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg, NULL);
    }
}

void device_api_send_min_scene(const char *dsuid, Scene scene)
{
    vdsm_report(LOG_DEBUG, "device api send min scene: on device %s call min scene %d\n", dsuid, scene);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationCallMinScene submsg =
            VDCAPI__VDSM__NOTIFICATION_CALL_MIN_SCENE__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        submsg.scene = scene;
        submsg.has_scene = 1;

        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_CALL_MIN_SCENE;
        msg.vdsm_send_call_min_scene = &submsg;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg, NULL);
    }
}

void device_api_identify(const char *dsuid)
{
    vdsm_report(LOG_DEBUG, "device api identify: identify (blink) device %s\n",
           dsuid);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationIdentify submsg =
            VDCAPI__VDSM__NOTIFICATION_IDENTIFY__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_IDENTIFY;
        msg.vdsm_send_identify = &submsg;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg, NULL);
    }
}

void device_api_set_outval(const char *dsuid, uint8_t channel, uint16_t outval)
{
    vdsm_report(LOG_DEBUG, "device api set outval: on device %s set value %d\n",
           dsuid, outval);

    vdc_client_t *vdc_client = NULL;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        // convert output data to vDSM API Range
        double output = 0;
        if (!convert_dsmval_to_vdcapival(channel, outval, &output)) {
            vdsm_report(LOG_ERR, "vdc api request set value: failed get conversion parameters\n");
            return;
        }

        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationSetOutputChannelValue submsg =
                VDCAPI__VDSM__NOTIFICATION_SET_OUTPUT_CHANNEL_VALUE__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        msg.vdsm_send_output_channel_value = &submsg;
        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_SET_OUTPUT_CHANNEL_VALUE;

        submsg.apply_now = true;
        submsg.has_apply_now = TRUE;

        submsg.channel = channel;
        submsg.has_channel = TRUE;

        submsg.value = output;
        submsg.has_value = TRUE;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg,  NULL);
    } else {
        vdsm_report(LOG_WARNING, "vdc api (set outval). device with dsuid: %s "
                            "not found\n", dsuid);
    }
}

bool device_api_set_zone(const char *dsuid, uint16_t zoneId)
{
    bool success = FALSE;
    vdsm_report(LOG_DEBUG, "device api set zone: on device %s set zone %d\n", dsuid, zoneId);

    vdc_client_t *vdc_client = NULL;
    Vdcapi__Message *msg = NULL;
    Vdcapi__VdsmRequestSetProperty *submsg = NULL;
    Vdcapi__PropertyElement *elemZone = NULL;

    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        msg = malloc(sizeof(Vdcapi__Message));
        if (!msg) {
            goto cleanup_set_zone;
        }
        vdcapi__message__init(msg);

        if (proputil_set_property_new(dsuid, &submsg) != 0) {
            goto cleanup_set_zone;
        }
        msg->vdsm_request_set_property = submsg;
        msg->type = VDCAPI__TYPE__VDSM_REQUEST_SET_PROPERTY;

        // Zone Id
        if (proputil_create_property_int(&elemZone, "zoneID", zoneId) != 0) {
            goto cleanup_set_zone;
        }

        if (proputil_set_property_add(submsg, elemZone) != 0) {
            goto cleanup_set_zone;
        }

        // message will be freed when response is received
        if (vdc_api_send_message(vdc_client, msg, NULL) == VDC_API_OK) {
            success = TRUE;
        }
    }
    return success;

cleanup_set_zone:
    vdsm_report(LOG_ERR, "vdc api request set zone: failed to allocate memory\n");
    free(msg);
    proputil_set_property_free(submsg);
    return FALSE;
}

static void device_set_group(vdc_client_t *vdc_client, const char *dsuid,
                             uint8_t groupId, int set)
{
    Vdcapi__Message *msg = NULL;
    Vdcapi__VdsmRequestSetProperty *submsg = NULL;
    Vdcapi__PropertyElement* elemOutputSettings = NULL;
    Vdcapi__PropertyElement* elemGroups = NULL;
    Vdcapi__PropertyElement* elemGroupId = NULL;

    msg = malloc(sizeof(Vdcapi__Message));
    if (!msg) {
        vdsm_report(LOG_ERR, "vdc api request set property: failed to allocate "
                        "memory\n");
        return;
    }
    vdcapi__message__init(msg);

    if (proputil_set_property_new(dsuid, &submsg) != 0) {
        goto cleanup_set_group;
    }
    msg->vdsm_request_set_property = submsg;
    msg->type = VDCAPI__TYPE__VDSM_REQUEST_SET_PROPERTY;

    if (proputil_create_property_empty(&elemOutputSettings, "outputSettings") != 0) {
        goto cleanup_set_group;
    }
    if (proputil_set_property_add(submsg, elemOutputSettings) != 0) {
        proputil_property_recursive_free(elemOutputSettings);
        goto cleanup_set_group;
    }

    // outputGroups
    if (proputil_create_property_empty(&elemGroups, "groups") != 0) {
        goto cleanup_set_group;
    }
    if (proputil_add_property_element(elemOutputSettings, elemGroups) != 0) {
        proputil_property_recursive_free(elemGroups);
        goto cleanup_set_group;
    }

    // Group id and value
    char buf[4];
    sprintf(buf, "%i", groupId);
    if (proputil_create_property_bool(&elemGroupId, buf, (bool)set) != 0) {
        goto cleanup_set_group;
    }
    if (proputil_add_property_element(elemGroups, elemGroupId) != 0) {
        proputil_property_recursive_free(elemGroupId);
        goto cleanup_set_group;
    }

    // message will be freed when response is received
    vdc_api_send_message(vdc_client, msg, NULL);
    return;

cleanup_set_group:
    vdsm_report(LOG_ERR, "vdc api request set property: failed to allocate memory\n");
    free(msg);
    proputil_set_property_free(submsg);
}

void device_api_add_group(const char *dsuid, uint8_t groupId)
{
    vdsm_report(LOG_DEBUG, "device api add group: on device %s add group %d\n", dsuid, groupId);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        device_set_group(vdc_client, dsuid, groupId, 1);
    }
}

void device_api_remove_group(const char *dsuid, uint8_t groupId)
{
    vdsm_report(LOG_DEBUG, "device api remove group: on device %s remove group %d\n", dsuid, groupId);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        device_set_group(vdc_client, dsuid, groupId, 0);
    }
}

void device_api_get_parameter(const char* dsuid, uint8_t paramBank, uint8_t paramOffset)
{
    vdc_client_t *vdc_client;
    uint16_t deviceId = Datamodel_GetDeviceIdFromDsuid(dsuid);
    vdsm_report(LOG_DEBUG, "device api get parameter. device: %s, bank: %d, offset: %d\n", dsuid, paramBank, paramOffset);

    switch (paramBank) {
    case 1:
        switch (paramOffset) {
        case FID: {
            // function id
            if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
                request_cache_t *cache;

                // get button number of device
                cache = (request_cache_t*) malloc(sizeof(request_cache_t));
                if (cache == NULL)
                {
                    vdsm_report(LOG_ERR, "could not allocate memory for multi "
                                    "property request cache\n");
                    return;
                }
                cache->context = REQUEST_CONTEXT_GET_PROPERTY_FID;
                cache->offset = paramOffset;
                cache->count = 0;
                cache->result = 0x0100;
                cache->channel = 0; // only channel 0
                vdc_api_send_get_property(vdc_client, dsuid, "primaryGroup",
                                          cache);
            }
            return;
        }

        default: {
            if ((paramOffset >= SensorTableStart) && (paramOffset < SensorTableLast)) {
                // check sensors
                uint16_t retVal = 0;
                uint8_t sensorIndex, type, broadcast, pushConvert;
                uint32_t pollInterval;

                sensorIndex = (paramOffset - SensorTableStart);
                if (Datamodel_GetDeviceSensor(deviceId, sensorIndex, &type, &pollInterval, &broadcast, &pushConvert)) {
                    retVal |= (type << 8);
                    /* pollInterval in seconds can not be represented in upstream message */
                    retVal |= 0;
                    retVal |= (broadcast? 0x08 : 0);
                    retVal |= (pushConvert? 0x10 : 0);
                } else {
                    retVal = 0xffff;
                }
                /* create long upstream event */
                LongUpstreamEvt *filteredUpEvt = Q_NEW(LongUpstreamEvt, LONG_UPSTREAM_SIG);
                filteredUpEvt->source = deviceId;
                filteredUpEvt->mainqualifier = 0;
                filteredUpEvt->subqualifier = paramOffset / 2;
                filteredUpEvt->data = retVal;
                filteredUpEvt->quality = 62;
                filteredUpEvt->flags = 0;
                /* post event */
                QF_publish((QEvent*)filteredUpEvt);
                vdsm_report(LOG_DEBUG, "get sensor: published event. value: 0x%04x\n", retVal);
                return;
            } else if ((paramOffset >= BinaryInputTableStart) && (paramOffset < BinaryInputTableLast)) {
                // check binary input
                uint16_t retVal = 0;
                uint8_t targetGroup, type, buttonId, binInpIndex, binInpOffset;

                binInpIndex = ((paramOffset - BinaryInputTableStart) / 3);
                binInpOffset = ((paramOffset - BinaryInputTableStart) % 3);
                if (Datamodel_GetDeviceBinaryInput(deviceId, binInpIndex, &targetGroup, &type, &buttonId)) {
                    if (binInpOffset  == 0) {
                        retVal = (type << 8) | targetGroup;
                    } else if (binInpOffset == 1) {
                        retVal = type;
                        if (Datamodel_GetDeviceBinaryInput(deviceId, binInpIndex + 1, &targetGroup, &type, &buttonId)) {
                            retVal |= (buttonId << 12);
                        } else {
                            retVal |= 0x0100;
                        }
                    } else {
                        retVal = (buttonId << 4);
                        if (Datamodel_GetDeviceBinaryInput(deviceId, binInpIndex + 1, &targetGroup, &type, &buttonId)) {
                            retVal |= (targetGroup << 8);
                        } else {
                            retVal |= 0x01;
                        }
                    }
                    /* create long upstream event */
                    LongUpstreamEvt *filteredUpEvt = Q_NEW(LongUpstreamEvt, LONG_UPSTREAM_SIG);
                    filteredUpEvt->source = deviceId;
                    filteredUpEvt->mainqualifier = 0;
                    filteredUpEvt->subqualifier = paramOffset / 2;
                    filteredUpEvt->data = retVal;
                    filteredUpEvt->quality = 62;
                    filteredUpEvt->flags = 0;
                    /* post event */
                    QF_publish((QEvent*)filteredUpEvt);
                    vdsm_report(LOG_DEBUG, "get binary input: published event. value: 0x%04x\n", retVal);
                    return;
                } else {
                    vdsm_report(LOG_DEBUG, "get binary input: no binary input found for index %d\n", binInpIndex);
                }
            } else if (paramOffset >= OpcTableStart) {

                uint8_t index = paramOffset-OpcTableStart;

                uint8_t loChannelId = 0;
                uint8_t hiChannelId = 0;

                index = index & 0xfe;
                if (index == 0) {
                    loChannelId = 0xff;
                    Datamodel_GetOpcChannelFromIndex(deviceId,index,&hiChannelId);
                } else {
                    Datamodel_GetOpcChannelFromIndex(deviceId,index-1,&loChannelId);
                    Datamodel_GetOpcChannelFromIndex(deviceId,index,&hiChannelId);
                }

                uint16_t output = (loChannelId +(hiChannelId <<8));

                /* create long upstream event */
                LongUpstreamEvt *filteredUpEvt = Q_NEW(LongUpstreamEvt, LONG_UPSTREAM_SIG);
                filteredUpEvt->source = deviceId;
                filteredUpEvt->mainqualifier = 0;
                filteredUpEvt->subqualifier = paramOffset / 2;
                filteredUpEvt->data = output;
                filteredUpEvt->quality = 62;
                filteredUpEvt->flags = 0;
                /* post event */
                QF_publish((QEvent*)filteredUpEvt);
                vdsm_report(LOG_DEBUG, "get OpcTable Channel: published event.index: 0x%04x value: 0x%04x\n", index, output);
                return;
            }
        }
        }
        break;
    case 3:
        switch (paramOffset) {
        case MODE: {
            if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
                request_cache_t *cache;

                cache = (request_cache_t*) malloc(sizeof(request_cache_t));
                if (cache == NULL)
                {
                    vdsm_report(LOG_ERR, "could not allocate memory for multi "
                                    "property request cache\n");
                    return;
                }

                cache->context = REQUEST_CONTEXT_GET_PROPERTY_MODE;
                cache->offset = paramOffset;
                cache->count = 0;
                cache->result = 0;
                cache->channel = 0;
                vdc_api_send_get_property(vdc_client, dsuid, "outputSettings",
                                          cache);
            }
            return;
        }
        case LTMODE:
        case PBCHANNEL:
        case PBGROUP: {
            if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
                request_cache_t *cache;
                cache = (request_cache_t*) malloc(sizeof(request_cache_t));
                if (cache == NULL)
                {
                    vdsm_report(LOG_ERR, "could not allocate memory for multi "
                                    "property request cache\n");
                    return;
                }

                cache->offset = paramOffset;
                cache->channel = 0;           // Channel 0 only for this request

                if (paramOffset== LTMODE) {
                    cache->context = REQUEST_CONTEXT_GET_PROPERTY_LTMODE;
                } else if (paramOffset== PBCHANNEL) {
                    cache->context = REQUEST_CONTEXT_GET_PROPERTY_PBCHANNEL;
                } else if (paramOffset== PBGROUP) {
                    cache->context = REQUEST_CONTEXT_GET_PROPERTY_PBGROUP;
                }
                vdc_api_send_get_property(vdc_client, dsuid,
                                          "buttonInputSettings", cache);
            }
            return;
        }
        }
        break;
    case 64:
        switch (paramOffset) {
        case 0: {
            // output value
            if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
                request_cache_t *cache;

                cache = (request_cache_t*) malloc(sizeof(request_cache_t));
                if (cache == NULL)
                {
                    vdsm_report(LOG_ERR, "could not allocate memory for multi "
                                    "property request cache\n");
                    return;
                }

                cache->context = REQUEST_CONTEXT_GET_PROPERTY_OUTPUT_VALUE;
                cache->channel = 0; // only 0 channel, if addressed with bank 64
                cache->offset = paramOffset; // only 0 supported

                int data[1];
                data[0] = 0;
                vdc_api_send_get_properties_indexed(vdc_client, dsuid,
                                                    "channelStates", data, 1, cache);
            }
            return;
        }
        }
        break;
    case 128:
        if (paramOffset < 256) {
            if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
                // scene value/configuration
                request_cache_t *cache;

                cache = (request_cache_t*) malloc(sizeof(request_cache_t));
                if (cache == NULL)
                {
                    vdsm_report(LOG_ERR, "could not allocate memory for multi "
                                    "property request cache\n");
                    return;
                }

                cache->context = REQUEST_CONTEXT_GET_PROPERTY_SCENES;
                cache->offset = paramOffset;
                cache->channel = 0;

                int data[2];
                data[0] = (paramOffset & 0x7E);
                data[1] = (paramOffset & 0x7E) +1;
                vdc_api_send_get_properties_indexed(vdc_client, dsuid,
                                                    "scenes", data, 2, cache);
            }
            return;
        }
    }

    vdsm_report(LOG_INFO, "device api get parameter. offset %d on bank %d not "
                     "found\n", paramOffset, paramBank);
    /* create long upstream event */
    LongUpstreamEvt *filteredUpEvt = Q_NEW(LongUpstreamEvt, LONG_UPSTREAM_SIG);
    filteredUpEvt->source = deviceId;
    filteredUpEvt->source = Datamodel_GetDeviceIdFromDsuid(dsuid);
    filteredUpEvt->mainqualifier = 0;
    filteredUpEvt->subqualifier = paramOffset / 2;
    filteredUpEvt->data = 0;
    filteredUpEvt->quality = 62;
    filteredUpEvt->flags = 0;
    /* post event */
    QF_publish((QEvent*)filteredUpEvt);
    vdsm_report(LOG_DEBUG, "device api get parameter. published event with default "
                      "value 0\n");
}

void device_api_send_dim_channel(const char *dsuid, uint8_t channel, OPCOperation mode, uint8_t area)
{
    vdsm_report(LOG_DEBUG, "device api send dim channel: on device %s set channel %d to mode %d\n", dsuid, channel, mode);

    vdc_client_t *vdc_client;
    if (vdc_api_get_vdc_from_lookup_table(dsuid, &vdc_client)) {
        Vdcapi__Message msg = VDCAPI__MESSAGE__INIT;
        Vdcapi__VdsmNotificationDimChannel submsg =
            VDCAPI__VDSM__NOTIFICATION_DIM_CHANNEL__INIT;

        char *dsuids[1];
        dsuids[0] = (char *) dsuid;
        submsg.dsuid = dsuids;
        submsg.n_dsuid = 1;

        submsg.channel = channel;
        submsg.has_channel = 1;

        submsg.area = area;
        submsg.has_area = 1;

        switch (mode)
        {
            case OPC_INC:  submsg.mode = 1;  break;
            case OPC_DEC:  submsg.mode = -1; break;
            case OPC_STOP: submsg.mode = 0;  break;
        }
        submsg.has_mode = 1;

        submsg.area = 0;
        submsg.has_area = 1;

        msg.type = VDCAPI__TYPE__VDSM_NOTIFICATION_DIM_CHANNEL;
        msg.vdsm_send_dim_channel = &submsg;

        // static message, not stored for response tracking
        vdc_api_send_message(vdc_client, &msg, NULL);
    }
}

