/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Authors: Christian Hitz, digitalSTROM AG <christian.hitz@digitalstrom.com>
             Michael Tross, digitalSTROM AG <michael.tross@digitalstrom.com>
             Reto Flütsch, digitalSTROM AG <reto.fluetsch@digitalstrom.com>

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.
    

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef UTIL_H_
#define UTIL_H_

#include "types.h"
#include <string.h>
#include <syslog.h>
#include <digitalSTROM/dsuid.h>

#include <stdio.h>
#include <google/protobuf-c/protobuf-c.h>

#if defined HAVE_BACKTRACE && defined HAVE_BACKTRACE_SYMBOLS
#define print_backtrace() vdsm_util_print_backtrace()
#else
#define print_backtrace()
#endif
double get_timestamp(void);

struct dS485_server;

typedef struct _socketClient {
    int fd;                   /* client socket descriptor */
    struct dS485_server *server;
    double active;            /* last activity timestamp */
} SocketClient;

void socketDetach(SocketClient* cl);

int openVdcSessionSocket(const char *vdc_url, const char *vdc_port);
char *sock2ip(int fd);

void vdsm_init_report(int debugLevel);
void vdsm_report(int errlevel, const char *fmt, ... );
void vdsm_report_extraLevel(int errlevel, int maxErrlevel, const char *fmt, ... );
bool vdsm_util_parse_uri(const char *uri, char **host, char **port);
#if defined HAVE_BACKTRACE && defined HAVE_BACKTRACE_SYMBOLS
void vdsm_util_print_backtrace();
#endif/* backtrace */
void *unaligned_memcpy(void *dst, const void *src, size_t n);
void protobufMessagePrint(const ProtobufCMessage *aMessageP, int aIndent);
void convert_string_to_upper(char *data, size_t len);
int dsuid_to_string_upper(const dsuid_t *dsuid, char *buf);

#endif /* UTIL_H_ */
